import sys


try:
    import psycopg2
    HAS_PSYCOPG2 = True
except ImportError:
    psycopg2 = None
    HAS_PSYCOPG2 = False


if sys.version < '3':
    text_type = unicode
    binary_type = str
    string_types = basestring
else:
    text_type = str
    binary_type = bytes
    string_types = str


try:
    from django.contrib.gis.geos.libgeos import geos_version_info
    HAS_GEOS = geos_version_info()['version'] >= '3.3.0'
except (ImportError, OSError):
    HAS_GEOS = False



from functools import partial


class Blueprint(object):
    def __init__(self, model, fields=None, pre_save=None, post_save=None, seed=None):
        from .faker_factory import factory

        self.factory = factory

        self._model = model
        self._fields = fields or {}
        self.seed = seed
        self.pre_save = pre_save
        self.post_save = post_save

        self.pk = -1
        super(Blueprint, self).__init__()

    def fields(self, **kwargs):
        self._fields = kwargs
        return self

    def make_one(self, fields=None, pre_save=None, post_save=None, seed=None, iteration=None):
        _fields = self._fields.copy()
        if fields:
            _fields.update(fields)
        if seed is None:
            seed = self.seed

        if pre_save is None:
            pre_save = self.pre_save

        if post_save is None:
            post_save = self.post_save

        return self.factory.make_one(self._model, _fields, pre_save, post_save, seed, iteration)

    def make(self, fields=None, pre_save=None, post_save=None, seed=None, quantity=None):
        _fields = self._fields.copy()
        if fields:
            _fields.update(fields)
        if seed is None:
            seed = self.seed

        if pre_save is None:
            pre_save = self.pre_save

        if post_save is None:
            post_save = self.post_save

        return self.factory.make(self._model, _fields, pre_save, post_save, seed, quantity)

    def build(self, fields=None, pre_save=None, seed=None, quantity=None, make_fks=False):
        _fields = self._fields.copy()
        if fields:
            _fields.update(fields)
        if seed is None:
            seed = self.seed

        if pre_save is None:
            pre_save = self.pre_save

        return self.factory.build(self._model, _fields, pre_save, seed, quantity, make_fks)

    def m(self, pre_save=None, post_save=None, seed=None, quantity=None):
        make = partial(self.make, pre_save=pre_save, post_save=post_save, seed=seed, quantity=quantity)

        def fn(**kwargs):
            return make(fields=kwargs)

        return fn

    def b(self, pre_save=None, seed=None, quantity=None, make_fks=False):
        build = partial(self.build, pre_save=pre_save, seed=seed, quantity=quantity, make_fks=make_fks)

        def fn(**kwargs):
            return build(fields=kwargs)

        return fn



from functools import partial
from django.db import models

from .compat import string_types

from . import field_mappings


class Evaluator(object):

    def __init__(self, faker, factory, iteration):
        self.faker = faker
        self.factory = factory
        self.iteration = iteration
        super(Evaluator, self).__init__()

    def evaluate(self, value):
        from .blueprint import Blueprint

        if isinstance(value, Blueprint):
            return value.make_one(iteration=self.iteration)
        if callable(value):
            if value.__name__ == '<lambda>':
                return value(self.iteration, self.faker)
            else:
                return value()
        if isinstance(value, string_types):
            return value.format(self.iteration, self.faker)
        return value

    def evaluate_fake(self, resolver, field):
        if callable(resolver[0]):
            func = partial(resolver[0], self.faker, field)
        else:
            func = getattr(self.faker, resolver[0])
        return func(*resolver[1], **resolver[2])

    def fake_value(self, model, field):
        if field.blank and not isinstance(field, models.BooleanField):
            return ''

        if isinstance(field, models.ForeignKey):
            return self.factory.make_one(field.related_model, iteration=self.iteration)

        if field.name in field_mappings.mappings_names:
            return self.evaluate_fake(field_mappings.mappings_names[field.name], field)

        for field_class, fake in field_mappings.mappings_types.items():
            if isinstance(field, field_class):
                return self.evaluate_fake(fake, field)

        model_name = '%s.%s' % (model._meta.app_label, model._meta.model_name)
        raise ValueError('Cant generate a value for model `%s` field `%s`' % (model_name, field.name))



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django import VERSION as django_version

from django.db import models, migrations
from autoslug import AutoSlugField

try:
    from django.contrib.gis.geos.libgeos import geos_version_info
    HAS_GEOS = geos_version_info()['version'] >= '3.3.0'
except (ImportError, OSError):
    HAS_GEOS = False

if HAS_GEOS:
    import django.contrib.gis.db.models.fields

from django.contrib.postgres.operations import HStoreExtension
import django.contrib.postgres.fields
import django.contrib.postgres.fields.hstore


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [HStoreExtension()]
    operations += [
        migrations.CreateModel(
            name='Chef',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('slug', AutoSlugField()),
                ('first_name', models.CharField(max_length=60)),
                ('last_name', models.CharField(max_length=60)),
                ('uuid_id', models.UUIDField()),
                ('email_address', models.EmailField(max_length=60)),
                ('twitter_profile', models.URLField(max_length=60)),
            ],
        ),
        migrations.CreateModel(
            name='Topping',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=60)),
            ],
        ),
        migrations.CreateModel(
            name='Pizza',
            fields=[
                ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
                ('name', models.CharField(max_length=50)),
                ('price', models.DecimalField(null=True, max_digits=4, decimal_places=2)),
                ('gluten_free', models.BooleanField(default=False)),
                ('vegan', models.BooleanField()),
                ('description', models.TextField(blank=True)),
                ('thickness', models.CharField(max_length=50, choices=[(0, b'thin'), (1, b'thick'), (2, b'deep dish')])),
                ('backed_on', models.DateTimeField()),
                ('chef', models.ForeignKey(to='tests.Chef', related_name='invented_pizzas')),
                ('critic', models.ForeignKey(to='tests.Chef', null=True, related_name='reviewed_pizzas')),
                ('toppings', models.ManyToManyField(to='tests.Topping')),
                ('rating', models.PositiveSmallIntegerField()),
                ('unique_comment', models.TextField(unique=True)),
            ],
        ),
    ]

    if HAS_GEOS:
        pizzeria_fields = [
            ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
            ('hq', django.contrib.gis.db.models.fields.PointField(srid=4326)),
            ('directions', django.contrib.gis.db.models.fields.LineStringField(srid=4326)),
            ('floor_plan', django.contrib.gis.db.models.fields.PolygonField(srid=4326)),
            ('locations', django.contrib.gis.db.models.fields.MultiPointField(srid=4326)),
            ('routes', django.contrib.gis.db.models.fields.MultiLineStringField(srid=4326)),
            ('delivery_areas', django.contrib.gis.db.models.fields.MultiPolygonField(srid=4326)),
            ('all_the_things', django.contrib.gis.db.models.fields.GeometryCollectionField(srid=4326)),
        ]
        if django_version >= (1, 9, 0):
            pizzeria_fields.append(
                ('rast', django.contrib.gis.db.models.fields.RasterField(srid=4326)),
            )
        operations += [
            migrations.CreateModel(
                name='Pizzeria',
                fields=pizzeria_fields,
            )
        ]

    specialtypizza_fields = [
        ('id', models.AutoField(verbose_name='ID', serialize=False, auto_created=True, primary_key=True)),
        ('toppings', django.contrib.postgres.fields.ArrayField(base_field=models.CharField(max_length=20), size=4)),
        ('metadata', django.contrib.postgres.fields.hstore.HStoreField()),
        ('price_range', django.contrib.postgres.fields.IntegerRangeField()),
        ('sales', django.contrib.postgres.fields.BigIntegerRangeField()),
        ('available_on', django.contrib.postgres.fields.DateTimeRangeField()),
        ('season', django.contrib.postgres.fields.DateRangeField()),
    ]

    if django_version >= (1, 9, 0):
        specialtypizza_fields.append(
            ('nutritional_values', django.contrib.postgres.fields.JSONField()),
        )

    operations += [
        migrations.CreateModel(
            name='SpecialtyPizza',
            fields=specialtypizza_fields,
        ),
    ]



from functools import partial
import random

from django.apps import apps
from django.conf import settings
from django.contrib.auth import get_user_model
from django.db import models
from django.db.models.fields import NOT_PROVIDED

from faker import Factory as FakerFactory

from .compat import string_types
from .exceptions import ForeignKeyError
from .lazy import Lazy
from .utils import get_model_fields, language_to_locale
from .values import Evaluator


user_model = get_user_model()

locale = language_to_locale(settings.LANGUAGE_CODE)


class Empty(object):
    pass


class Factory(object):

    def __init__(self, fake=None):
        self.fake = fake or FakerFactory.create(locale)

    def _get_model(self, model):
        if isinstance(model, string_types):
            model = apps.get_model(*model.split('.'))
        return model

    def seed(self, seed, set_global=False):
        self.fake.seed(seed)
        if set_global:
            random.seed(seed)

    def blueprint(self, *args, **kwargs):
        from .blueprint import Blueprint
        return Blueprint(*args, **kwargs)

    def build_one(self, model, fields=None, pre_save=None, seed=None, make_fks=False, iteration=None):
        if fields is None:
            fields = {}

        if pre_save is None:
            pre_save = []

        if seed:
            fake = FakerFactory.create(locale)
            fake.seed(seed)
        else:
            fake = self.fake

        evaluator = Evaluator(fake, factory=self, iteration=iteration)

        model = self._get_model(model)
        instance = model()
        m2ms = {}
        lazies = []

        for _field_name, model_field in get_model_fields(model):
            if _field_name.endswith('_id') and model_field.is_relation:
                field_name = _field_name.split('_id')[0]
            else:
                field_name = _field_name

            if isinstance(model_field, models.AutoField):
                continue

            if field_name not in fields and (model_field.null or model_field.default != NOT_PROVIDED):
                    continue

            if field_name not in fields and isinstance(model_field, models.ManyToManyField):
                continue

            if isinstance(model_field, models.ForeignKey):
                value = fields.get(field_name, Empty)

                if not make_fks and ((value == Empty) or (value and value.pk is None)):
                    raise ForeignKeyError(
                        "Field {} is a required ForeignKey, but the related {}.{} model"
                        " doesn't have the necessary primary key.".format(
                            field_name,
                            model_field.related_model._meta.app_label,
                            model_field.related_model._meta.model_name,
                        )
                    )

            if field_name in fields:
                value = evaluator.evaluate(fields[field_name])
            else:
                if model_field.choices:
                    value = fake.random_element(model_field.choices)[0]
                else:
                    value = evaluator.fake_value(model, model_field)

            if isinstance(value, Lazy):
                lazies.append((field_name, value))
                continue

            if isinstance(model_field, models.ForeignKey):
                field_name += '_id'
                value = value.pk if value else None

            if isinstance(model_field, models.ManyToManyField):
                m2ms[field_name] = value
            # special case for user passwords
            if model == user_model and field_name == 'password':
                instance.set_password(value)
            else:
                if field_name not in m2ms:
                    setattr(instance, field_name, value)

        for field_name, lazy in lazies:
            value = getattr(instance, lazy.name)
            if callable(value):
                value = value(*lazy.args, **lazy.kwargs)
            setattr(instance, field_name, value)

        for func in pre_save:
            func(instance)

        return instance, m2ms

    def build(self, model, fields=None, pre_save=None, seed=None, quantity=None, make_fks=False):
        if fields is None:
            fields = {}

        if quantity:
            return [self.build_one(model, fields, pre_save, seed, make_fks, i)[0] for i in range(quantity)]
        else:
            return self.build_one(model, fields, pre_save, seed, make_fks)[0]

    def make_one(self, model, fields=None, pre_save=None, post_save=None, seed=None, iteration=None):
        if fields is None:
            fields = {}

        if post_save is None:
            post_save = []

        instance, m2ms = self.build_one(model, fields, pre_save, seed, make_fks=True, iteration=iteration)
        instance.save()

        for field, relateds in m2ms.items():
            setattr(instance, field, relateds)

        for func in post_save:
            func(instance)
        return instance

    def get_or_make(self, model, lookup=None, fields=None, pre_save=None, post_save=None, seed=None):
        if lookup is None:
            lookup = {}
        if fields is None:
            fields = {}

        model = self._get_model(model)
        try:
            return model.objects.get(**lookup), False
        except model.DoesNotExist:
            attrs = {}
            attrs.update(lookup)
            attrs.update(fields)
            return self.make_one(model, fields=attrs, pre_save=None, post_save=None, seed=None), True

    def g_m(self, model, lookup=None, pre_save=None, post_save=None, seed=None):
        build = partial(self.get_or_make, model=model, lookup=lookup, pre_save=pre_save, seed=seed)

        def fn(**kwargs):
            return build(fields=kwargs)

        return fn

    def make(self, model, fields=None, pre_save=None, post_save=None, seed=None, quantity=None):
        if fields is None:
            fields = {}
        if quantity:
            return [self.make_one(model, fields, pre_save, post_save, seed, i) for i in range(quantity)]
        else:
            return self.make_one(model, fields, pre_save, post_save, seed)

    def m(self, model, pre_save=None, post_save=None, seed=None, quantity=None):
        make = partial(self.make, model=model, pre_save=pre_save, post_save=post_save, seed=seed, quantity=quantity)

        def fn(**kwargs):
            return make(fields=kwargs)

        return fn

    def b(self, model, pre_save=None, seed=None, quantity=None, make_fks=False):
        build = partial(self.build, model=model, pre_save=pre_save, seed=seed, quantity=quantity, make_fks=make_fks)

        def fn(**kwargs):
            return build(fields=kwargs)

        return fn


factory = Factory()



from collections import OrderedDict

import sys

from django.db import models
from django import VERSION as django_version

from .compat import HAS_PSYCOPG2, HAS_GEOS
from . import fakes


"""
This module maps fields to functions generating values.

It first tries by looking at the field's class, then falls back to some
special-cased names.

Values are 3-tuples composed of ``(<function>, <args>, <kwargs>)``.

When ``<function>`` is a string, it's assumed to be a faker provider. Whenever
``faker`` doesn't provide a suitable provider, we ship our own function. They
are defined in ``django_fakery.fakes``.
"""

mappings_types = OrderedDict([
    (models.BigIntegerField, ('random_int', [], {'min': -sys.maxsize, 'max': sys.maxsize})),
    (models.BinaryField, (fakes.random_bytes, [1024], {})),
    (models.BooleanField, ('pybool', [], {})),
    (models.DateField, (lambda faker, field: faker.date_time().date(), [], {})),
    (models.DateTimeField, ('date_time', [], {})),
    (models.DecimalField, (fakes.decimal, [], {})),
    (models.EmailField, ('email', [], {})),
    (models.FileField, ('file_name', [], {})),
    (models.FilePathField, ('file_name', [], {})),
    (models.FloatField, ('pyfloat', [], {})),
    (models.ImageField, ('file_name', [], {'extension': 'jpg'})),
    (models.IntegerField, ('pyint', [], {})),
    (models.IPAddressField, ('ipv4', [], {})),
    (models.GenericIPAddressField, ('ipv4', [], {})),
    (models.PositiveIntegerField, ('random_int', [], {'max': 2147483647})),
    (models.PositiveSmallIntegerField, ('random_int', [], {'max': 32767})),
    (models.SlugField, (fakes.slug, [3], {})),
    (models.SmallIntegerField, ('random_int', [], {'min': -32768, 'max': 32767})),
    (models.TextField, (lambda faker, field: field.unique and faker.pystr(max_chars=2700) or faker.paragraph(), [], {})),
    (models.TimeField, (lambda faker, field: faker.date_time().time(), [], {})),
    (models.URLField, ('url', [], {})),
    (models.CharField, (lambda faker, field: field.unique and faker.pystr(max_chars=field.max_length) or faker.word()[:field.max_length], [], {})),
    (models.DurationField, ('time_delta', [], {})),
    (models.UUIDField, ('uuid4', [], {})),
])

if django_version < (1, 10, 0):
    mappings_types.update({
        models.CommaSeparatedIntegerField: (fakes.comma_sep_integers, [], {}),
    })

if HAS_GEOS:
    from django.contrib.gis.db import models as geo_models

    mappings_types.update({
        geo_models.PointField: (fakes.point, (), {'srid': 4326}),
        geo_models.LineStringField: (fakes.linestring, (), {'srid': 4326}),
        geo_models.PolygonField: (fakes.polygon, (), {'srid': 4326}),
        geo_models.MultiPointField: (fakes.multipoint, (), {'srid': 4326}),
        geo_models.MultiLineStringField: (fakes.multilinestring, (), {'srid': 4326}),
        geo_models.MultiPolygonField: (fakes.multipolygon, (), {'srid': 4326}),
        geo_models.GeometryCollectionField: (fakes.geometrycollection, (), {'srid': 4326}),
    })

    if django_version >= (1, 9, 0):
        mappings_types.update({
            geo_models.RasterField: (fakes.gdal_raster, [], {'srid': 4326}),
        })

if HAS_PSYCOPG2:
    from django.contrib.postgres import fields as pg_fields

    mappings_types.update({
        pg_fields.ArrayField: (fakes.array, [], {}),
        pg_fields.HStoreField: ('pydict', [10, True, 'str'], {}),
        pg_fields.IntegerRangeField: (fakes.integerrange, [], {'min': -2147483647, 'max': 2147483647}),
        pg_fields.BigIntegerRangeField: (fakes.integerrange, [], {'min': -sys.maxsize, 'max': sys.maxsize}),
        pg_fields.FloatRangeField: (fakes.floatrange, [], {}),
        pg_fields.DateTimeRangeField: (fakes.datetimerange, [], {}),
        pg_fields.DateRangeField: (fakes.daterange, [], {}),
    })
    if django_version >= (1, 9, 0):
        mappings_types.update({
            pg_fields.JSONField: (fakes.random_dict, [], {}),
        })

mappings_names = {
    'name': (lambda faker, field: field.unique and faker.pystr(max_chars=field.max_length or 2700) or faker.word()[:field.max_length], [], {}),  # `name` is too generic to assume it's a person
    'slug': (fakes.slug, [3], {}),  # `name` is too generic to assume it's a person
    'first_name': ('first_name', [], {}),
    'last_name': ('last_name', [], {}),
    'full_name': ('full_name', [], {}),
    'email': ('email', [], {}),
    'created': ('date_time_between', [], {'start_date': '-30d', 'end_date': '30d'}),
    'created_at': ('date_time_between', [], {'start_date': '-30d', 'end_date': '30d'}),
    'updated': ('date_time_between', [], {'start_date': '-30d', 'end_date': '30d'}),
    'updated_at': ('date_time_between', [], {'start_date': '-30d', 'end_date': '30d'}),
}



VERSION = '1.7.0'

from django.utils.functional import SimpleLazyObject
from django.utils.module_loading import import_string
from .blueprint import Blueprint
from .lazy import Lazy

factory = SimpleLazyObject(lambda: import_string('django_fakery.faker_factory.factory'))



def language_to_locale(language):
    """
    Converts django's `LANGUAGE_CODE` settings to a proper locale code.
    """
    tokens = language.split('-')
    if len(tokens) == 1:
        return tokens[0]
    return "%s_%s" % (tokens[0], tokens[1].upper())


def get_model_fields(model):
    fields = list(model._meta._forward_fields_map.items())
    for m2m in model._meta.many_to_many:
        fields.append((m2m.name, m2m))
    return fields



import os
from setuptools import find_packages, setup


VERSION = '1.7.0'


def read(fname):
    try:
        with open(os.path.join(os.path.dirname(__file__), fname)) as fh:
            return fh.read()
    except IOError:
        return ''

setup(
    name='django-fakery',
    version=VERSION,
    url='https://github.com/fcurella/django-fakery/',
    author='Flavio Curella',
    author_email='flavio.curella@gmail.com',
    description='A model instances generator for Django',
    long_description=read('README.rst'),
    license='MIT',
    packages=find_packages(exclude=['*.tests']),
    platforms=["any"],
    classifiers=[
        'Development Status :: 4 - Beta',
        'Framework :: Django',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 2',
        'Programming Language :: Python :: 2.7',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3.3',
        'Programming Language :: Python :: 3.4',
    ],
    install_requires=[
        "Faker>=0.7.0,<0.8.0",
        "Django>=1.8",
        "django-autoslug==1.9.3",
    ],
    test_suite='django_fakery.tests.runtests.runtests',
)
